/*
 * Device Driver for DataTranslation DT2801
 *
 *
 */

#ifndef _DT2801_ADC_H
#define _DT2801_ADC_H 1

#ifdef __KERNEL__

#include "dt2801.h"

/* A/D-converter */

#define DT2801_NUM_ADCS 1
#define DT2801_ADC_CHANNELS 8

/* Minor device numbers */
#define DT2801_ADCX     0x1f /* the generic adc-device */


#define DT2801_ADC_UNIPOLAR 0
#define DT2801_ADC_BIPOLAR  1

#define DT2801_ADC_DIFFERENTIAL 0
#define DT2801_ADC_SINGLE_ENDED 1


extern struct file_operations dt2801_adc_fops;
extern int dt2801_adc_init(void);
extern int dt2801_adc_cleanup(void);
extern int dt2801_adc_reset(void);
extern int dt2801_adc_open(struct inode *inode, struct file *filp);

typedef struct {
        int bits;
	int polarity;
	int inputmode;
	int nchannels;

        /* 'gain', 'startch', 'endch' as set by the last
	   set_adc_params command */
        int gain;

        int startch;
        int endch;
        int conv;

        /* XXX  for DMA-transfers*/
	int last;
	int currnt;
	int running;
	int cnt;

	int dmabufsize;
	void* dmabuf;
} ADC_state;

typedef struct {
        Dt2801_t* board;
	int mode;
	int exttrigger;
	int extclock;
	int continous;
        int startch;
        int endch;
        int gain;
        int conv;
} ADCP;

#endif /* __KERNEL__ */

typedef struct {
    int startch;
    int endch;
    int gain;
    int conv;
} Dt2801_ADCP_t;

#define DT2801_ADC_IMMEDIATE  1
#define DT2801_ADC_BLOCK      2

#define DT2801_IOCTL_ADC_SET_PARAMS _IOW(DT2801_MAGIC, 0x11, Dt2801_ADCP_t)
#define DT2801_IOCTL_ADC_BLOCKSTART _IO(DT2801_MAGIC, 0x12)
#define DT2801_IOCTL_ADC_BLOCKSTOP  _IO(DT2801_MAGIC, 0x13)
#define DT2801_IOCTL_ADC_RESIDUE    _IOR(DT2801_MAGIC, 0x14, int)
#define DT2801_IOCTL_ADC_SETEXTCLOCK _IOW(DT2801_MAGIC, 0x15, int)
#define DT2801_IOCTL_ADC_SETCONTINOUS _IOW(DT2801_MAGIC, 0x16, int)
#define DT2801_IOCTL_ADC_GET_POLARITY   _IOR(DT2801_MAGIC, 0x17, int)
#define DT2801_IOCTL_ADC_SET_GAIN       _IOW(DT2801_MAGIC, 0x18, int)
#define DT2801_IOCTL_ADC_SETEXTTRIGGER _IOW(DT2801_MAGIC, 0x19, int)


#endif /* _dt2801_adc_h_ */
