/*
 * Device Driver for DataTranslation DT2801
 *
 * Digital-Analog-Converter subsystem
 *
 *
 */


/* Available: 
      DAC0/1 immediate mode */

/* not Available: 
      block modes */

/* I really dont't know how to integrate the mode, where
   both DACs work together. As its own device it is not like
   _one_ DAC, because it has two channels that are completley
   independent, except that their outputs are set simultanously.

   On the other hand, to make a block-mode (with DMA) work, data
   for both channels must be given.

   Has to wait, and is currently disabled. */
   
  
#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/errno.h>
#include <linux/malloc.h>

#include <asm/io.h>
#include <asm/segment.h>
#include <asm/system.h>
#include <asm/dma.h>

#ifndef MODULE
#error "driver must be compiled as a module"
#endif

#include "dt2801.h"
#include "dac.h"

static DAC_state dac_cf;

static int dac_write_imm(struct inode *inode, struct file *filp,
			    const char* buffer, int count)
{	
	short data1, data2;
	DACP* dacp = (DACP*) filp->private_data;
	int dacnr = dacp->dacnr;

	if (dacnr == 0 || dacnr == 1) {
		if (count != 2) {
			return -(EINVAL);
		}
		data1 = get_user((short*)buffer);
		dt2801_writecommand(DT_C_WRITE_DAIM);
		dt2801_writedata(dacnr);
		dt2801_writedata2(data1);
	} else if (dacnr == 2) {
		if (count != 4) {
			return -(EINVAL);
		}
		data1 = get_user((short*)buffer);
		data2 = get_user((short*)(buffer+sizeof(short)));
		dt2801_writecommand(DT_C_WRITE_DAIM);
		dt2801_writedata(dacnr);
		dt2801_writedata2(data1);
		dt2801_writedata2(data2);
        } else {
		printk("dt2801: dac_write_imm: invalid dacnr %d.\n", dacnr);
                return -1;
	}
	
	return count;
}

/* How should that be handled ?
   I think we need all the data in a buffer before we write it
   (with DMA). How to deal with continous mode ? */
static int dac_write_block(struct inode *inode, struct file *filp,
			    const char* buffer, int count)
{
	return 0;
}

static int dac_write(struct inode *inode, struct file *filp,
			    const char* buffer, int count)
{

	DACP* dacp = (DACP*) filp->private_data;

	switch (dacp->mode) {
	case DT2801_DAC_IMMEDIATE: {
		return dac_write_imm(inode, filp, buffer, count);
	} break;

	case DT2801_DAC_BLOCK: {
		return dac_write_block(inode, filp, buffer, count);
	}

	}

	return 0;
}

static int dac_setparams(int dacnr, int conv)
{
		dt2801_writecommand(DT_C_SET_DA);
		dt2801_writedata(dacnr);
		dt2801_writedata2(conv);

		dac_cf.dacnr = dacnr;
		dac_cf.conv = conv;

		return 0;
}

static int dac_ioctl(struct inode *inode, struct file *filp,
			    unsigned int cmd, unsigned long arg)
{
	DACP* dacp = (DACP*) filp->private_data;

	switch(cmd) {
	case DT2801_IOCTL_DAC_SET_PARAMS: {
		Dt2801_DACP_t dacparams;
                int dacnr, conv;

                dacnr = dacp->dacnr;
                conv = 0; /* XXX */

		memcpy_fromfs(&dacp, (char*) arg, sizeof(dacparams));

		dac_setparams(dacnr, conv);

		return dt2801_checkforerror();
	}
	break;
	case DT2801_IOCTL_DAC_GET_POLARITY: {
                int dacnr;
                int stat;
                
                dacnr = dacp->dacnr;
                stat = verify_area(VERIFY_WRITE, (char *)arg, sizeof(int));
                if (stat != 0) {
                        return stat;
                }
                put_user(dac_cf.dacs[dacnr].polarity, (int *)arg);

		return 0;
	}
	break;
	case DT2801_IOCTL_DAC_GET_FULLSCALE: {
                int dacnr;
                int stat;

		dacnr = dacp->dacnr;
                stat = verify_area(VERIFY_WRITE, (char *)arg, sizeof(int));
                if (stat != 0) {
                        return stat;
                }
                put_user(dac_cf.dacs[dacnr].fullscale, (int *)arg);
		return 0;
	}
	break;	case DT2801_IOCTL_DAC_SET_MODE: {

                dacp->mode = (int)arg;
		return 0;
	}
	break;
	default:
		return dt2801_ioctl(inode, filp, cmd, arg);
	}
	
	return 0;
}

int dt2801_dac_reset(void)
{

        /* Both DAC's are set to 0 after power on or reset, which
	   gives 0V for unipolar and -fullscale for bipolar
	   configuration. */

	/* Anything to be done here ? */

        /* Maybe set them to 0V f bipolar ? */
	/*
	data = 2048; nr = 0;
	dt2801_writecommand(DT_C_WRITE_DAIM);
	dt2801_writedata(nr);
	dt2801_writedata2(data);
	data = 2048; nr = 1;
	dt2801_writecommand(DT_C_WRITE_DAIM);
	dt2801_writedata(nr);
	dt2801_writedata2(data);
	*/

	return 0;
}

int dt2801_dac_cleanup(void)
{
	return 0;
}


static int dac_parse_polarity(char *str)
{
	int polarity;

	if (!strcmp(str, "unipolar")) {
		polarity  = DT2801_DAC_UNIPOLAR;
	} else if (!strcmp(str, "bipolar")) {
		polarity  = DT2801_DAC_BIPOLAR;
	} else {
		return -(EINVAL);
	}

        return polarity;
}

/* Is there a scanf() in the kernel ? */
/* Is there anything special about using floatingpoint variables 
   in the kernel ? */
/* How does insmod deal with more arguments than array elements ?*/

/* ! modules-2.0.0 has bug that arrays of (char*) are not properly
   set. Seems to be fixed in 2.1.8 and above. 2.1.13 is ok, checked
   the source */
#define DT2801_DAC_MAX_FSNAMES 3
static char* dac_fs_names[DT2801_DAC_MAX_FSNAMES] = {"10.0", "5.0", "2.5"};

static int dac_parse_fullscale(char *str)
{
        int i;

        for (i=0; i<DT2801_DAC_MAX_FSNAMES; i++) {
		if (!strcmp(str, dac_fs_names[i])) {
			return i;
		}
	}
	return -(EINVAL);
}


/* May be given as parameters at load-time */
static char* dac_fullscale[] =  { "10.0", "10.0"};
static char* dac_polarity[] =  {"bipolar", "bipolar"};

int dt2801_dac_init()
{
        int i;
        DAC_t* dac;

        dac_cf.conv = 0;
        dac_cf.dacnr = -1;

        for (i=0; i<DT2801_NDACS; i++) {

		dac = &dac_cf.dacs[i];

		dac->polarity  = dac_parse_polarity(dac_polarity[i]);
		if (dac->polarity < 0) {
			printk("dt2801: invalid value for 'dac_polarity': "
			       "%s\n", dac_polarity[i]);
			return -(EINVAL);
		}

		dac->fullscale = dac_parse_fullscale(dac_fullscale[i]);
		if (dac->fullscale < 0) {
			printk("dt2801: invalid value for 'dac_fullscale': "
			       "%s\n", dac_fullscale[i]);
			return -(EINVAL);
		}

		if (dac->fullscale == 2 &&
		    dac->polarity == DT2801_DAC_UNIPOLAR) {
			printk("dt2801: unipolar/%sV not available\n",
			       dac_fs_names[dac->fullscale]);
			return -(EINVAL);
		}

		dac_cf.dacs[i].bits = 12;
		dac_cf.dacs[i].usecount = 0;
	}

        for (i=0; i<DT2801_NDACS; i++) {
		printk("dt2801: DAC%d %s, %s\n", i,
		       dac_polarity[i], dac_fullscale[i]);
	}

	return 0;
}

static void dt2801_dac_release(struct inode *inode, struct file *filp)
{
	DACP* dacp = (DACP*) filp->private_data;
        int dacnr = dacp->dacnr;

	if (dacnr == 0 || dacnr == 1) {
		dac_cf.dacs[dacp->dacnr].usecount--;
	} else if (dacnr == 2) {
		dac_cf.dacs[0].usecount--;
		dac_cf.dacs[1].usecount--;
	}
        kfree(dacp);

	dt2801_release(inode, filp);
}


static struct file_operations dac_fops = {
	NULL,                /* seek */
	NULL,                /* read */
	dac_write,           /* write */
	NULL,                /* readdir */
	NULL,                /* select */
	dac_ioctl,           /* ioctl */
	NULL,                /* mmap */
	dt2801_dac_open,     /* open */
	dt2801_dac_release   /* release */
};

int dt2801_dac_open(struct inode *inode, struct file *filp)
{
	int minor = MINOR(inode->i_rdev);
        int dacnr;
	DACP* dacp;

	DT_PRINTKF("minor = %d\n", minor);

        if (minor < 0 || minor >= 2) { /* XXX  > 2*/
		return -(ENODEV);
        }

        dacnr = minor & 0x3;
	DT_PRINTKF("dacnr = %d\n", dacnr);
        if (dacnr < 0 || dacnr > 2) {
		printk("dt2801: dac_open: invalid dacnr = %d.\n", dacnr);
		return -(EINVAL);
        }

        if (dacnr == 0 || dacnr == 1) {
		if (dac_cf.dacs[dacnr].usecount != 0) {
			printk("dt2801: dac_open: "
			       "dacnr = %d not free.\n", dacnr);
			return -(EBUSY);
		}
		dac_cf.dacs[dacnr].usecount++;

	} else if (dacnr == 2) {
		if (dac_cf.dacs[0].usecount != 0 ||
		    dac_cf.dacs[1].usecount != 0) {
			printk("dt2801: dac_open: dac 0+1 not free.\n");
			return -(EBUSY);
		}
		dac_cf.dacs[0].usecount++;
		dac_cf.dacs[1].usecount++;
	} else {
		printk("dt2801: dac_open: invalid dacnr = %d.\n", dacnr);
		return -(EINVAL);
	}

	dacp = (DACP*) kmalloc(sizeof(DACP), GFP_KERNEL);
	if (dacp == NULL) {
		return -(ENOMEM);
	}
        dacp->dacnr = dacnr;
        dacp->mode = DT2801_DAC_IMMEDIATE;
        dacp->exttrigger = 0;
	filp->private_data = dacp;
	filp->f_op = &dac_fops;

	return 0;
}

