/*
 * Device Driver for DataTranslation DT2801
 *
 *
 */

#include <linux/errno.h>
#include <linux/errno.h>
#include <linux/malloc.h>
#include <linux/fcntl.h>

#include <asm/io.h>
#include <asm/segment.h>
#include <asm/system.h>
#include <asm/dma.h>

#ifndef MODULE
#error "dt2801_dio must be compiled as a module"
#endif

#include "dt2801.h"
#include "dio.h"


static DIO_state dio_cf;

static int set_port_mode(int port, int mode) 
{
	if (port < 0 || port > 1) {
		printk("dt2801: set_port_mode: invalid port #%d\n", port);
		return -(EINVAL);
	}

        switch (mode) {
        case DT2801_DIO_PORTIN:
		dt2801_writecommand(DT_C_SET_DIGIN);
		dt2801_writedata(port);
                dio_cf.ports[port].mode = DT2801_DIO_PORTIN;
		break;
        case DT2801_DIO_PORTOUT:
		dt2801_writecommand(DT_C_SET_DIGOUT);
		dt2801_writedata(port);
                dio_cf.ports[port].mode = DT2801_DIO_PORTOUT;
		break;
        default:
		printk("dt2801: set_port_mode: invalid mode %d\n", mode);
		return -(EINVAL);
	}

	return 0;
}
 
static int dio_read(struct inode *inode, struct file *filp,
			   char* buffer, int count)
{
	DIO* diop = (DIO*) filp->private_data;
	int minor = MINOR(inode->i_rdev);
	int port = diop->port;

	printk("dt2801: dio_read: minor = %d  port = %d.\n", minor, port);


	return 0;
}

static int dio_read2(struct inode *inode, struct file *filp,
		     char* buffer, int count)
{
	DIO* diop = (DIO*) filp->private_data;
	int minor = MINOR(inode->i_rdev);
	int port = diop->port;

	printk("dt2801: dio_read2: minor = %d  port = %d.\n", minor, port);


	return 0;
}

static int dio_write(struct inode *inode, struct file *filp,
		     const char* buffer, int count)
{
	DIO* diop = (DIO*) filp->private_data;
        int databyte = 0;


	int minor = MINOR(inode->i_rdev);
	int port = diop->port;

	printk("dt2801: dio_write: minor = %d  port = %d.\n", minor, port);

	dt2801_writecommand(DT_C_WRITE_DIG);
	dt2801_writedata(port);
	dt2801_writedata(databyte);

	return count;
}

static int dio_write2(struct inode *inode, struct file *filp,
		      const char* buffer, int count)
{
	DIO* diop = (DIO*) filp->private_data;
        int databyte = 0;


	int minor = MINOR(inode->i_rdev);
	int port = diop->port;

	printk("dt2801: dio_write2: minor = %d  port = %d.\n", minor, port);

	dt2801_writecommand(DT_C_WRITE_DIG);
	dt2801_writedata(port);
	dt2801_writedata2(databyte);

	return count;
}

static int dio_ioctl(struct inode *inode, struct file *filp,
			    unsigned int cmd, unsigned long arg)
{
	DIO* diop = (DIO*) filp->private_data;
	int minor = MINOR(inode->i_rdev);
	int port = diop->port;

	printk("dt2801: dio_ioctl: minor = %d  port = %d.\n", minor, port);

	switch (cmd) {
	default:
		return dt2801_ioctl(inode, filp, cmd, arg);
	}
	return 0;
}

static int dio_ioctl_g(struct inode *inode, struct file *filp,
			    unsigned int cmd, unsigned long arg)
{
	DIO* diop = (DIO*) filp->private_data;
	int minor = MINOR(inode->i_rdev);
	int port = diop->port;

	printk("dt2801: dio_ioctl_g: minor = %d  port = %d.\n", minor, port);

	switch (cmd) {
	case DT2801_IOCTL_DIO_SET_LINES: {
	}
	break;
	default:
		return dt2801_ioctl(inode, filp, cmd, arg);
	}
	return 0;
}



/* After board-reset or power-up */
int dt2801_dio_reset(void)
{
        int i;

        for (i=0; i<dio_cf.nports; i++) {
		dio_cf.ports[i].mode = DT2801_DIO_PORTIN;
	}

	return 0;
}

int dt2801_dio_cleanup(void)
{

	return 0;
}

int dt2801_dio_init(void)
{
        int i;

        dio_cf.usecount = 0;
        dio_cf.nports = DT2801_DIO_NPORTS;

        for (i=0; i<dio_cf.nports; i++) {
		dio_cf.ports[i].usecount = 0;
		dio_cf.ports[i].mode = DT2801_DIO_PORTUNDEF;
		dio_cf.ports[i].lines = 0;
	}
	printk("dt2801: DIO %d ports\n", dio_cf.nports);

	return 0;
}

static void dio_release(struct inode *inode, struct file *filp)
{
	DIO* diop = (DIO*) filp->private_data;
	int i;

	printk("dt2801: dio_release\n");
	dio_cf.ports[diop->port].usecount--;

        kfree(diop);
	for (i=0; i<dio_cf.nports; i++) {
		printk("dt2801: dio_usecount[%d] = %d.\n", 
		       i, dio_cf.ports[i].usecount);
	}
	dt2801_release(inode, filp);
}

static void dio_release2(struct inode *inode, struct file *filp)
{
	DIO* diop = (DIO*) filp->private_data;
	int i;

	printk("dt2801: dio_release2\n");
	dio_cf.ports[0].usecount--;
	dio_cf.ports[1].usecount--;

        kfree(diop);
	for (i=0; i<dio_cf.nports; i++) {
		printk("dt2801: dio_usecount[%d] = %d.\n", 
		       i, dio_cf.ports[i].usecount);
	}
	dt2801_release(inode, filp);
}

static struct file_operations dio_fops_r = {
	NULL,                /* seek */
	dio_read,            /* read */
	NULL,                /* write */
	NULL,                /* readdir */
	NULL,                /* select */
	NULL,                /* ioctl */
	NULL,                /* mmap */
	dt2801_dio_open,     /* open */ 
	dio_release          /* release */
};

static struct file_operations dio_fops_w = {
	NULL,                /* seek */
	NULL,                /* read */
	dio_write,           /* write */
	NULL,                /* readdir */
	NULL,                /* select */
	NULL,                /* ioctl */
	NULL,                /* mmap */
	dt2801_dio_open,     /* open */ 
	dio_release          /* release */
};

static struct file_operations dio_fops_r2 = {
	NULL,                /* seek */
	dio_read2,           /* read */
	NULL,                /* write */
	NULL,                /* readdir */
	NULL,                /* select */
	NULL,                /* ioctl */
	NULL,                /* mmap */
	dt2801_dio_open,     /* open */ 
	dio_release2         /* release */
};

static struct file_operations dio_fops_w2 = {
	NULL,                /* seek */
	NULL,                /* read */
	dio_write2,          /* write */
	NULL,                /* readdir */
	NULL,                /* select */
	NULL,                /* ioctl */
	NULL,                /* mmap */
	dt2801_dio_open,     /* open */ 
	dio_release2         /* release */
};

/* A D-I/O port can be opened either for reading or writing. */
int dt2801_dio_open(struct inode *inode, struct file *filp)
{
        int stat = 0;
	int minor = MINOR(inode->i_rdev);
	int mode;
        int i;

	DIO* diop;

	printk("dt2801: dio_open: minor = % d.\n", minor);

	diop = (DIO*) kmalloc(sizeof(DIO), GFP_KERNEL);
	if (diop == NULL) {
		return -(ENOMEM);
	}

	mode = filp->f_flags & O_ACCMODE;

	if (mode == O_RDWR) {
		stat = -(EPERM);
		goto fail1;
	}

	switch (minor) {
        case DT2801_DIO0:
        case DT2801_DIO1: {
		int port = minor & 0x3;

		printk("dt2801: dio_open: port = % d.\n", port);

		if (mode == O_RDONLY) {
			if (dio_cf.ports[port].mode == DT2801_DIO_PORTOUT) {
				stat = -(EBUSY);
				goto fail1;
			}
			set_port_mode(port, DT2801_DIO_PORTIN);
			diop->mode = DT2801_DIO_PORTIN;
			filp->f_op = &dio_fops_r;
		} else { /*O_WRONLY */
			if (dio_cf.ports[port].usecount != 0) {
				stat = -(EBUSY);
				goto fail1;
			}
			set_port_mode(port, DT2801_DIO_PORTOUT);
			diop->mode = DT2801_DIO_PORTOUT;
			filp->f_op = &dio_fops_w;
		} 
		dio_cf.ports[port].usecount++;
		
		diop->port = port;
		break;
        }
        case DT2801_DIO2: {
		int port = minor & 0x3;

		printk("dt2801: dio_open: port = % d.\n", port);
		if (mode == O_RDONLY) {
			for (i=0; i<dio_cf.nports; i++) {
				if (dio_cf.ports[i].mode == DT2801_DIO_PORTOUT) {
					stat = -(EBUSY);
					goto fail1;
				}
			}
			set_port_mode(0, DT2801_DIO_PORTIN);
			set_port_mode(1, DT2801_DIO_PORTIN);
			diop->mode = DT2801_DIO_PORTIN;
			filp->f_op = &dio_fops_r2;
		} else { /*O_WRONLY */
			for (i=0; i<dio_cf.nports; i++) {
				if (dio_cf.ports[i].usecount != 0) {
					stat = -(EBUSY);
					goto fail1;
				}
			}
			set_port_mode(0, DT2801_DIO_PORTOUT);
			set_port_mode(1, DT2801_DIO_PORTOUT);
			diop->mode = DT2801_DIO_PORTOUT;
			filp->f_op = &dio_fops_w2;
		} 
		dio_cf.ports[0].usecount++;
		dio_cf.ports[1].usecount++;
		
		diop->port = port;

		break;
	}
        case DT2801_DIOX: {
		printk("dt2801: dio_open: generic device not implemented.\n");
		stat = -(ENODEV);
		goto fail1;

		/* filp->f_op->ioctl = dio_ioctl_g; */

		break;
	}
        default: {
		int line = minor & 0xf;

		if (minor >= 32 && minor <48) {
			printk("dt2801: dio_open: line = % d.\n", line);
			printk("dt2801: dio_open: single-line devices not "
			       "implemented.\n");
			stat = -(ENODEV);
			goto fail1;
			if (mode == O_RDONLY) {
			} else { /*O_WRONLY */
			}

		} else {
			stat = -(ENODEV);
			goto fail1;
		}
	}
	}

	filp->private_data = diop;

	for (i=0; i<dio_cf.nports; i++) {
		printk("dt2801: dio_usecount[%d] = %d\n.", 
		       i, dio_cf.ports[i].usecount);
	}
	return 0;

fail1:
	kfree(diop);
        return stat;
}


