/*
 * Device Driver for DataTranslation DT2801
 *
 *
 */

#ifndef _DT2801_DIO_H
#define _DT2801_DIO_H 1
#ifdef __KERNEL__

#define DT2801_DIO_NPORTS 2 
#define DT2801_DIO_PORTSIZE 8 

/* Minor device numbers */
#define DT2801_DIO0     0x30  /* port 0, 8-bit */
#define DT2801_DIO1     0x31  /* port 1, 8-bit */
#define DT2801_DIO2     0x32  /* port 0 and 1 treated as one 16-bit port */
#define DT2801_DIOX     0x3f  /* the generic dio-device */


/* To manage 2 8-bit ports */

#define DT2801_DIO_PORTUNDEF -1
#define DT2801_DIO_PORTIN 0 
#define DT2801_DIO_PORTOUT 1 

typedef unsigned short uint16;

typedef struct {
	int usecount;
	int mode;
	int lines; /* lower 8 bits only */
} DIO_port;

typedef struct {
	int usecount;
	int nports;
        DIO_port ports[DT2801_DIO_NPORTS];
} DIO_state;

typedef struct {
	uint16 lines;
	int port;
	int mode;
	int exttrigger;
} DIO;

extern struct file_operations dt2801_dio_fops;
extern int dt2801_dio_init(void);
extern int dt2801_dio_cleanup(void);
extern int dt2801_dio_reset(void);
extern int dt2801_dio_open(struct inode *inode, struct file *filp);

#endif /* __KERNEL__ */

#define DT2801_IOCTL_DIO_SET_LINES _IOW(DT2801_MAGIC, 0x31, uint16)

#endif /* _DT2801_DIO_H */
